//
//  TftMAnimMap.java
//  UlimateRetro
//
//  Created by Billy Spelchan on Sun Oct 27 2002.
//  Copyright (c) 2002 Spelchan Software. All rights reserved.
//
package com.blazinggames.retro.modern.search;

// Java imports
import java.awt.*;

// Spelchan imports (sscl)
import com.blazinggames.urcl.data.*;
import com.blazinggames.urcl.ui.*;
import com.blazinggames.urcl.ui.sprite.*;
import com.blazinggames.urcl.ui.playfield.*;
//import spelchan.ui.comp.*;

// retro imports
import com.blazinggames.retro.shared.search.tftm.*;

public class TftMAnimMap
extends Map
{
// *********************
// ***** CONSTANTS *****
// *********************
    
    // TftM images
    private static final String TILESET = "12TILESET";
    private static final String TANKICONS = "12TANKICONS";

// *********************
// ***** VARIABLES *****
// *********************

    DisplayHandler _dh;
    TftMDisplay _display;
    
    ImageStripSprite _tankSprites[][];
    Sprite _highlight;
    Rectangle _placement;
    Rectangle _viewport = new Rectangle(0,0,400,400);

    
// ************************
// ***** CONSTRUCTORs *****
// ************************

    TftMAnimMap(DisplayHandler dh, TftMDisplay display, Rectangle r)
    {
        super();
        
        _dh = dh;
        _display = display;
        _placement = r;
        Config config = _dh.getConfig();
        ResMan res = _dh.getResMan();
        Container c = _dh.getContainer();

        _highlight = new Sprite();
        _highlight.setBounds(0,0, 24,24);
        _highlight.setBackground(Color.yellow);
        _highlight.setSolid(true);
        _tankSprites = new ImageStripSprite[4][9];
        res.addImage(TANKICONS, config.getStringProperty(TANKICONS, "tanks.gif"));
        for (int cntrPlayer = 0; cntrPlayer < 4; ++cntrPlayer)
        {
            for (int cntrTank = 0; cntrTank < 9; ++cntrTank)
            {
                _tankSprites[cntrPlayer][cntrTank] = new ImageStripSprite(c, res.getImage(TANKICONS), 20,20);
            }
        }
    }

    
// ********************************
// ***** METHODS from TftMMap *****
// ********************************

    public int fire(int id, int tankID)
    {
        int r = super.fire(id, tankID);

        if (r == 0)
        {
            _display.playSound(TftMDisplay.MISS_SOUND);
        }
        if (r == 1)
        {
            _display.playSound(TftMDisplay.HIT_SOUND);
        }
        if (r == 2)
        {
            _display.playSound(TftMDisplay.DESTROY_SOUND);
        }

        return r;
    }

    
// *******************************
// ***** METHODS - Rendering *****
// *******************************

    public synchronized void render(Graphics g, Component comp, int curPlayer, int curTank)
    {
        int cntrTank, cntrPlayer;

        updateTankSprites();

        _map.render(g, comp, _placement.x, _placement.y, _viewport);

        if ( (curPlayer >= 0) && (curPlayer < 4) && (curTank >= 0) && (curTank < 9) )
        {
            _highlight.move(_tankSprites[curPlayer][curTank].getX() - 2,
                            _tankSprites[curPlayer][curTank].getY() - 2 );
            _highlight.setVisible(_tankSprites[curPlayer][curTank].isVisible());
            _highlight.render(g, comp);
        }
        else
        {
            _highlight.setVisible(false);
        }
        
        for (cntrPlayer = 0; cntrPlayer < 4; ++cntrPlayer)
        {
            for (cntrTank = 0; cntrTank < 9; ++cntrTank)
            {
                _tankSprites[cntrPlayer][cntrTank].render(g, comp);
            }
        }
    }

    private void updateTankSprites()
    {
        int cntrPlayer, cntrTank;
        Tank t;
        ImageStripSprite ts;
        
        for (cntrPlayer = 0; cntrPlayer < 4; ++cntrPlayer)
        {
            for (cntrTank = 0; cntrTank < 9; ++cntrTank)
            {
                t = _tanks[cntrPlayer][cntrTank];
                ts = _tankSprites[cntrPlayer][cntrTank];

                ts.setVisible(t.getHealth() > 0);
                ts.move(t.getX() * 20 + _placement.x, t.getY() * 20 + _placement.y);
                ts.setFrame(cntrPlayer * 8 + t.getFacing());
            }
        }
    }

}
