//
//  TftMApplet.java
//  UlimateRetro
//
//  Created by spelchan on Mon Nov 19 2001.
//  Copyright (c) 2001 Spelchan Software. All rights reserved.
//
package com.blazinggames.retro.modern.search;

// Spelchan imports
import com.blazinggames.urcl.URCLApplet;
import com.blazinggames.urcl.ui.Display;

// Retro imports
import com.blazinggames.retro.modern.Title;

/**
 * An applet that runs the Tanks for the Memory (Modern Version) game.
 *
 * @author Billy D. Spelchan
 * @version 1.00.011019
*/
public class TftMApplet
extends URCLApplet 
{
// *********************
// ***** CONSTANTS *****
// *********************
    
    private static final String NAME = "Tanks for the Memory";
    private static final String VERSION = "1.10.040114";
	private static final String COPYRIGHT = "Copyright "+((char)169)+" 2001-2002,2004 Blazing Games Inc. All Rights Reserved.";
	
// *********************
// ***** VARIABLES *****
// *********************

    TftMDisplay _game;


// *****************************************
// ***** METHODS - From SpelchanApplet *****
// *****************************************

    /**
     * Prepares all the displays used by the applet. In this case, this
     * consists of the default error screen (handled by the super class),
     * the title screen, and the game display.
    */
    protected void initDisplays()
    {
        System.out.println(NAME + VERSION);
        System.out.println(COPYRIGHT);
        _titleDisplay = new TftMTitle("12");
        _titleDisplay.initialize(this);
        _game = new TftMDisplay(getCodeBase());
        _game.initialize(this);
    }
    
    /**
     * @return my copyright string
    */
    public String getAppletInfo()
    {
        return NAME + " Version " + VERSION + COPYRIGHT;
    }

    /**
     * Handles messages from the various display classes. Messages that we
     * take care of are as follows:
     * <UL>
     * <LI><B>LOADING #</B> Updates loaded %. # being percent loaded.</LI>
     * <LI><B>TFTM_START</B>Gets and passes settings to TftM display
     * and switches to the artillery display screen.</LI>
     * <LI><B>ATFTM_TITLE</B>Switches to the title screen</LI>
     * </UL>
     *
     * @param caller The display class that is making the call.
     * @param command The command that is to be executed.
    */
    public void displayEvent(Display caller, String command)
    {
        super.displayEvent(caller, command);
        
        if (command.startsWith("TFTM_START"))
        {
            TftMTitle tt = (TftMTitle)_titleDisplay;
            int mapid = (int)(command.charAt(command.length() - 1)) - 48;
            // querry TftMTitle class for settings and set them
            for (int cntr = 0; cntr < 4; ++cntr)
            {
                _game.setPlayerType(cntr, tt.getPlayerType(cntr));
            }
            _game.startGame(mapid);

            showDisplay(2);
        }
        else if (command.startsWith("TFTM_TITLE"))
        {
            showDisplay(1);
        }
    }

    /**
     * Handles switching the game screen to the indicated display ID.
     * Currently they are set up as follows:
     * <UL>
     * <LI>0 - Error screen</LI>
     * <LI>1 - Title Screen</LI>
     * <LI>2 - Tanks for the Memory Game Screen</LI>
     * </LI>
     *
     * @param id ID of display to switch to
     * @return the indicated display
    */
    protected Display getDisplay(int id)
    {
        if (id == 2)
            return _game;
        else
            return super.getDisplay(id);
    }

	public String getName()
	{
		return NAME;
	}
}
