//
//  TftMDisplay.java
//  UlimateRetro
//
//  Created by spelchan on Mon Nov 19 2001.
//  Copyright (c) 2001 Spelchan Software. All rights reserved.
//
package com.blazinggames.retro.modern.search;

// Java imports
import java.awt.*;
import java.io.IOException;
import java.net.URL;
import java.util.Vector;

// Spelchan Imports
import com.blazinggames.urcl.data.Config;
import com.blazinggames.urcl.ui.*;
import com.blazinggames.urcl.ui.sprite.*;
import com.blazinggames.urcl.ui.playfield.*;
import com.blazinggames.urcl.ui.comp.*;

// Retro imports
import com.blazinggames.retro.shared.search.tftm.*;

/**
 *
 * @author Billy D. Spelchan
 * @version 1.00.011019
*/
public class TftMDisplay
extends Display
{
// *********************
// ***** CONSTANTS *****
// *********************

    private static final Point DEF_PLACE = new Point (0,0);
    private static final Rectangle DEF_RECT = new Rectangle(0,0,0,0);

    // Sound support
    private static final String USE_SOUND = "12USESOUND";
    private static final String SOUND_IMAGE = "12SOUNDIMAGE";
    private static final String SOUND_RECT = "12SOUNDRECT";
    public static final String MISS_SOUND = "12MISSSOUND";
    public static final String HIT_SOUND = "12HITSOUND";
    public static final String DESTROY_SOUND = "12DESTROYSOUND";
    public static final String WIN_SOUND = "12WIN_SOUND";
    
// TftM Colors
    private static final String PLAYERCOLOR = "12PLAYERCOLOR";
    private static final String MAP = "12MAP";
    
// TftM images
    private static final String TILESET = "12TILESET";
    private static final String TANKICONS = "12TANKICONS";
    
// Tftm Specific configuration - component layout
    private static final String DAY = "12DAY";
    private static final String MESSAGE = "12MESSAGE";
    private static final String PLAYFIELD = "12PLAYFIELD";

    private static final String TANKID = "12TANKID";
    private static final String MOVEMENT = "12MOVEMENT";
    private static final String HEALTH = "12TANKHEATH";
    private static final String AMMO = "12TANKAMMO";

    private static final String CMD_LEFT = "12CMDLEFT";
    private static final String CMD_RIGHT = "12CMDRIGHT";
    private static final String CMD_ADVANCE = "12CMDADVANCE";
    private static final String CMD_RETREAT = "12CMDRETREAT";
    private static final String CMD_FIRE = "12CMDFIRE";
    private static final String CMD_DONE = "12CMDDONE";

    private static final String DIALOG_RECT = "12DIALOGRECT";
    private static final String DIALOG_PLAYER = "12DIALOGPLAYER";
    private static final String DIALOG_BASE = "12DIALOGBASE";
    private static final String DIALOG_FACTORIES = "12DIALOGFACTORIES";
    private static final String DIALOG_TOTAL = "12DIALOGTOTAL";
    private static final String DIALOG_CASH = "12DIALOGCASH";
    private static final String DIALOG_BUILD = "12DIALOGBUILDTANK";
    private static final String DIALOG_YES = "12DIALOGYES";
    private static final String DIALOG_NO = "12DIALOGNO";
    private static final String DIALOG_OK = "12DIALOGOK";

    // Buttons
    private static final int BTN_LEFT = 0;
    private static final int BTN_RIGHT = 1;
    private static final int BTN_ADVANCE = 2;
    private static final int BTN_RETREAT = 3;
    private static final int BTN_FIRE = 4;
    private static final int BTN_DONE = 5;
    private static final int BTN_YES = 6;
    private static final int BTN_NO = 7;
    private static final int BTN_OK = 8;
    private static final int TOTAL_BUTTONS = 9;

    private static final Font BIG_FONT = new Font("Serif", Font.BOLD, 20);
    private static final Font HUGE_FONT = new Font("Serif", Font.BOLD, 72);
    
// *********************
// ***** VARIABLES *****
// *********************

    java.util.Random _random;
    boolean _fullPaint;
    boolean _gameOver;
    boolean _waitForRender;
    int _winner;
    
    int _playerType[];
    TftMAnimMap _mapData[];
    Color _playerColors[];
    
    // Sound management
    boolean _useSound;
    Rectangle _soundRect;
    Toggle _soundToggle;

    // Layout
    SLabel _day;
    SLabel _message;
    Rectangle _playfieldRect;
    SLabel _tankID;
    SLabel _movement;
    SLabel _health;
    SLabel _ammo;
    TextButton _buttons[];

    // Dialog layout
    boolean _showDialog;
    Rectangle _dialog;
    SLabel _dlgPlayer;
    SLabel _dlgBase;
    SLabel _dlgFactories;
    SLabel _dlgTotal;
    SLabel _dlgCash;
    SLabel _dlgBuild;
    
    // Configuration options
    private int _startingMoney = 5000;
    private int _tankCost = 5000;
    private int _factoryProfit = 250;
    private int _perTurnMoney = 500;
    
    // Game
    URL _mapBase;
    TftMAnimMap _map;
    private int _playerMoney[] = new int[4];
	private TftMDumbAI _ai[] = new TftMDumbAI[4];
    private String _messageText;
    
    // Gameflow - Turn management
    int _curDay;
    int _curPlayer;
    int _curTank;

// ***********************
// ***** CONSTRUCTOR *****
// ***********************
    public TftMDisplay(URL base)
    {
        super();
        _mapBase = base;
        _random = new java.util.Random();
    }

// *****************************************
// ***** METHODS - Initialization aids *****
// *****************************************

    /**
     * prepares all the label fonts as well as loads in the player colors
    */
    private void prepareSprites()
    {
        Container c = _displayHandler.getContainer();
        _playerType = new int[4];
        
    }


    /**
     * Prepares the hall, which is simply a playfield so it can easily be scrolled.
    */
    private void preparePlayfield()
    {
        _playfieldRect = _config.getRectangleProperty(PLAYFIELD, DEF_RECT);
        _map = new TftMAnimMap(_displayHandler, this, _playfieldRect);
        _res.addImage(TILESET, _config.getStringProperty(TILESET, "x"));
        _map._tileRenderer.setImage(_res.getImage(TILESET));

        String mapname;
        URL url;
        _mapData = new TftMAnimMap[4];
        _playerColors = new Color[4];
        for (int cntr = 0; cntr < 4; ++cntr)
        {
            try
            {
                mapname = _config.getStringProperty(MAP+"["+cntr+"]", "res/tftm/basic.map");
                url = new URL(_mapBase, mapname);
                _mapData[cntr] = new TftMAnimMap(_displayHandler, this, _playfieldRect);
                _mapData[cntr].loadMap(url);
            }
            catch (IOException ioe)
            {
                System.err.println("ERROR LOADING MAP");
            }
            _playerColors[cntr] = _config.getColorProperty(PLAYERCOLOR+"["+cntr+"]", Color.white);
        }
    }

    private void prepareSounds()
    {
        _useSound = (_config.getIntProperty(USE_SOUND, 0) == 1);
        _res.addImage(SOUND_IMAGE, _config.getStringProperty(SOUND_IMAGE, "sound.gif"));
        _soundRect = _config.getRectangleProperty(SOUND_RECT, DEF_RECT);
    	_soundToggle = new Toggle(_displayHandler.getContainer(),
                    _res.getImage(SOUND_IMAGE), _soundRect.x, _soundRect.y, _soundRect.width, _soundRect.height, 2);
        if (_useSound)
        {
            _res.addAudioClip(MISS_SOUND, _config.getStringProperty(MISS_SOUND, "silent.au"));
            _res.addAudioClip(HIT_SOUND, _config.getStringProperty(HIT_SOUND, "silent.au"));
            _res.addAudioClip(DESTROY_SOUND, _config.getStringProperty(DESTROY_SOUND, "silent.au"));
            _res.addAudioClip(WIN_SOUND, _config.getStringProperty(WIN_SOUND, "silent.au"));
        }
    }

    /**
     * processes the layout
    */
    public void prepareLayout()
    {
        Container c = _displayHandler.getContainer();

        Rectangle r = _config.getRectangleProperty(DAY, DEF_RECT);
        _day = new SLabel(c, DAY, r);
        _day.setFont(c, BIG_FONT);
        _day.setAlignment(SLabel.ALIGN_CENTER);
        r = _config.getRectangleProperty(MESSAGE, DEF_RECT);
    	_message = new SLabel(c, MESSAGE, r);
        r = _config.getRectangleProperty(TANKID, DEF_RECT);
    	_tankID = new SLabel(c, TANKID, r);
        _tankID.setFont(c, BIG_FONT);
        _tankID.setAlignment(SLabel.ALIGN_CENTER);
        r = _config.getRectangleProperty(MOVEMENT, DEF_RECT);
    	_movement = new SLabel(c, MOVEMENT, r);
        r = _config.getRectangleProperty(HEALTH, DEF_RECT);
    	_health = new SLabel(c, HEALTH, r);
        r = _config.getRectangleProperty(AMMO, DEF_RECT);
    	_ammo = new SLabel(c, AMMO, r);

        _buttons = new TextButton[TOTAL_BUTTONS];
        r = _config.getRectangleProperty(CMD_LEFT, DEF_RECT);
        _buttons[BTN_LEFT] = new TextButton(c, "TURN LEFT", r);
        r = _config.getRectangleProperty(CMD_RIGHT, DEF_RECT);
        _buttons[BTN_RIGHT] = new TextButton(c, "TURN RIGHT", r);
        r = _config.getRectangleProperty(CMD_ADVANCE, DEF_RECT);
        _buttons[BTN_ADVANCE] = new TextButton(c, "ADVANCE", r);
        r = _config.getRectangleProperty(CMD_RETREAT, DEF_RECT);
        _buttons[BTN_RETREAT] = new TextButton(c, "RETREAT", r);
        _buttons[BTN_RETREAT].setVisible(false);
        r = _config.getRectangleProperty(CMD_FIRE, DEF_RECT);
    	_buttons[BTN_FIRE] = new TextButton(c, "FIRE", r);
        r = _config.getRectangleProperty(CMD_DONE, DEF_RECT);
        _buttons[BTN_DONE] = new TextButton(c, "DONE", r);
        
    	_dialog = _config.getRectangleProperty(DIALOG_RECT, DEF_RECT);
        r = _config.getRectangleProperty(DIALOG_PLAYER, DEF_RECT);
    	_dlgPlayer = new SLabel(c, DIALOG_PLAYER, r);
        _dlgPlayer.setFont(c, BIG_FONT);
        _dlgPlayer.setAlignment(SLabel.ALIGN_CENTER);
        r = _config.getRectangleProperty(DIALOG_BASE, DEF_RECT);
    	_dlgBase = new SLabel(c, DIALOG_BASE, r);
        _dlgBase.setAlignment(SLabel.ALIGN_CENTER);
        r = _config.getRectangleProperty(DIALOG_FACTORIES, DEF_RECT);
    	_dlgFactories = new SLabel(c, DIALOG_FACTORIES, r);
        _dlgFactories.setAlignment(SLabel.ALIGN_CENTER);
        r = _config.getRectangleProperty(DIALOG_TOTAL, DEF_RECT);
    	_dlgTotal = new SLabel(c, DIALOG_TOTAL, r);
        _dlgTotal.setAlignment(SLabel.ALIGN_CENTER);
        r = _config.getRectangleProperty(DIALOG_CASH, DEF_RECT);
    	_dlgCash = new SLabel(c, DIALOG_CASH, r);
        _dlgCash.setAlignment(SLabel.ALIGN_CENTER);
        r = _config.getRectangleProperty(DIALOG_BUILD, DEF_RECT);
    	_dlgBuild = new SLabel(c, DIALOG_BUILD, r);
        _dlgBuild.setFont(c, BIG_FONT);
        _dlgBuild.setAlignment(SLabel.ALIGN_CENTER);
        r = _config.getRectangleProperty(DIALOG_YES, DEF_RECT);
        _buttons[BTN_YES] = new TextButton(c, "Yes", r);
        r = _config.getRectangleProperty(DIALOG_NO, DEF_RECT);
        _buttons[BTN_NO] = new TextButton(c, "No", r);
        r = _config.getRectangleProperty(DIALOG_OK, DEF_RECT);
        _buttons[BTN_OK] = new TextButton(c, "Okay", r);
    }


// ***********************************
// ***** METHODS -  From Display *****
// ***********************************

    /**
     * Initializes the game (using private helper methods) by getting necessary
     * resources and setting up configurable variables.
     *
     * @param dh the controlling applet/application
    */
    public boolean initialize(DisplayHandler dh)
    {
        super.initialize(dh);
        dh.displayEvent(this, "LOADING 25");
        prepareSprites();
        dh.displayEvent(this, "LOADING 50");
        prepareSounds();
        preparePlayfield();
        dh.displayEvent(this, "LOADING 75");
        prepareLayout();
        dh.displayEvent(this, "LOADING 100");

        return true;
    }

    public boolean begin()
    {
        _fullPaint = true;
        return super.begin();
    }


    /**
     * Handles mouse movement. We may need to update the current guess variable
     * and if the game is over, the quit button needs messages.
     *
     * @param x X coordinate of mouse pointer
     * @param y Y coordinate of mouse pointer
     * @param b true if mouse button down
    */
    public void mouse_move(int x, int y, boolean b)
    {
        boolean repaint = false;
        
        for (int cntr = 0; cntr < TOTAL_BUTTONS; ++cntr)
        {
            repaint |= _buttons[cntr].mouse_move(x, y, b);
        }

        if (repaint)
            _displayHandler.refreshDisplay(this);
    }

    /**
     * Handles the mouse being clicked. If over the hallway (playfield) then
     * player may be starting to make a guess. If the game is over then we
     * have to pass messages to the quit button.
     *
     * @param x X coordinate of mouse pointer
     * @param y Y coordinate of mouse pointer
    */
    public void mouse_down(int x, int y)
    {
        boolean repaint = false;

        for (int cntr = 0; cntr < TOTAL_BUTTONS; ++cntr)
        {
            repaint |= _buttons[cntr].mouse_down(x, y);
        }
        repaint |= _soundToggle.mouse_down(x, y);
        
        if (repaint)
            _displayHandler.refreshDisplay(this);
    }

    /**
     * Handles the mouse button being released. This sends messages to the
     * scrollbar and (if game over) quit button components as well as handling
     * clicking on a door in the hallway playfield.
     *
     * @param x X coordinate of mouse pointer
     * @param y Y coordinate of mouse pointer
    */
    public void mouse_up(int x, int y)
    {
        boolean repaint = true;

        if (_gameOver)
        {
            _displayHandler.displayEvent(this, "TFTM_TITLE");
            return;
        }

        for (int cntr = 0; cntr < TOTAL_BUTTONS; ++cntr)
        {
            if (_buttons[cntr].mouse_up(x, y))
            {
                repaint = true;
                handleButton(cntr);
            }
        }
        repaint |= _soundToggle.mouse_up(x, y);
        
        if (repaint)
            _displayHandler.refreshDisplay(this);
    }

    /**
     * Ticks occur at a specified interval (100ms in my case).
     *
     * @param ms milliseconds between this click and the last one
    */
    public void tick(int ms)
    {
        boolean repaint = false;
        int cntr;

        for (cntr = 0; cntr < TOTAL_BUTTONS; ++cntr)
        {
            repaint |= _buttons[cntr].tick(ms);
        }
        repaint |= _soundToggle.tick(ms);
        
        if ( (_curPlayer >= 0) && (_curPlayer < 4) )
        {
            if ( (_playerType[_curPlayer] > 0) && (getRenderState() == false) )
            {
                if ( (_ai[_curPlayer].isDone() == false) && (_ai[_curPlayer].lastDay() == _curDay))
                {
                    _messageText = _ai[_curPlayer].continueTurn();
                    _curTank = _ai[_curPlayer].getCurrentTank();
                    if (_ai[_curPlayer].isDone())
                        nextPlayer();
                    else
                        setRenderState(true);
                }
                else if (_ai[_curPlayer].lastDay() == _curDay) // make sure has moved
                {
                    nextPlayer();
                }
                repaint = true;
            }
        }
        if (repaint)
        {
            _displayHandler.refreshDisplay(this);
        }
    }

    /**
     * draws the display screen to the indicated graphics context.
     *
     * @param g Graphics context to draw to
     * @param comp Component for awt calls
    */
    public synchronized void render(Graphics g, Component comp)
    {
        if (_gameOver)
        {
            g.setColor(_playerColors[_winner]);
        }
        else if ( (_curPlayer < 0) || (_curPlayer > 3) )
            g.setColor(Color.black);
        else
            g.setColor(_playerColors[_curPlayer]);
        g.fillRect(0,0,640,480);
        
        _map.render(g, comp, _curPlayer, _curTank);
        setInfo();
    	_day.render(g, comp);
    	_message.render(g, comp);
    	_tankID.render(g, comp);
    	_movement.render(g, comp);
    	_health.render(g, comp);
    	_ammo.render(g, comp);
        
        for (int cntr = 0; cntr < TOTAL_BUTTONS; ++cntr)
            _buttons[cntr].render(g, comp);
        if (_showDialog)
        {
            drawDialog(g, comp);
        }

        _soundToggle.render(g, comp);
        if (_soundToggle.getState() == 1)
        {
            g.setColor(Color.red);
            g.drawRect(_soundRect.x, _soundRect.y, _soundRect.width, _soundRect.height);
        }
        if (_gameOver)
        {
            drawGameOver(g, comp);
        }
        setRenderState(false);
    }

    private void setInfo()
    {
        Container c = _displayHandler.getContainer();
        _day.setText(c, " Day #" + _curDay);
        _message.setText(c, _messageText);

        if ((_curPlayer < 0) || (_curPlayer > 3) || (_curTank < 0) || (_curTank > 8))
        {
            _tankID.setText(c, " ");
            _movement.setText(c, " ");
            _health.setText(c, " ");
            _ammo.setText(c, " ");
            return;
        }

        Tank t = _map._tanks[_curPlayer][_curTank];
        
        _tankID.setText(c, " Tank #" + (_curTank + 1));
        _movement.setText(c, " Movement: " + t.getMovement() );
        _health.setText(c, " Health: " + t.getHealth() );
        _ammo.setText(c, " Ammo: " + t.getAmmo() );
    }


// **********************************
// ***** METHODS - Drawing aids *****
// **********************************
    
    private void drawGameOver(Graphics g, Component comp)
    {
//        g.setColor(_playerColors[_winner]);
//        g.fillRect(0,0, 640,480);
        Rectangle r = new Rectangle(0,0,640,480);
        String s = "Player " + (_winner + 1) + " won!";
        Font oldFont = g.getFont();
        g.setFont(HUGE_FONT);
        Point p = getTextPosition(g.getFontMetrics(), r, s, ALIGN_CENTER);
        drawOutlineText(g, s, p.x, p.y, _playerColors[_winner], Color.black);
        r.setBounds(0,240,640,240);
        s = "Click to return to menu...";
        g.setFont(BIG_FONT);
        p = getTextPosition(g.getFontMetrics(), r, s, ALIGN_CENTER);
        drawShadowText(g, s, p.x, p.y, Color.yellow, Color.black);
        g.setFont(oldFont);
    }
    
    private void drawDialog(Graphics g, Component c)
    {
        g.setColor(Color.black);
        g.fillRect(_dialog.x, _dialog.y, _dialog.width, _dialog.height);
    	_dlgPlayer.render(g, c);
    	_dlgBase.render(g, c);
    	_dlgFactories.render(g, c);
    	_dlgTotal.render(g, c);
    	_dlgCash.render(g, c);
    	_dlgBuild.render(g, c);

        _buttons[BTN_YES].render(g, c);
        _buttons[BTN_NO].render(g, c);
        _buttons[BTN_OK].render(g, c);
    }
    
// ********************************
// ***** METHODS - Sound aids *****
// ********************************

    /**
     * If sound is enabled gets the resource manager to play the indicated sound.
     *
     * @param key sound to play
    */
    public void playSound(String key)
    {
        if ( (_useSound) && (_soundToggle.getState() == 1) )
        {
            _res.playAudio(key);
        }
    }


// ***********************************************
// ***** METHODS - User Interface Management *****
// ***********************************************

    private void hideButtons()
    {
        for (int cntr = BTN_LEFT; cntr <= BTN_DONE; ++cntr)
        {
            _buttons[cntr].setVisible(false);
            _buttons[cntr].setActive(false);
        }
    }

    private void showButtons()
    {
        for (int cntr = BTN_LEFT; cntr <= BTN_DONE; ++cntr)
        {
            _buttons[cntr].setVisible(true);
            _buttons[cntr].setActive(true);
        }
        _buttons[BTN_RETREAT].setVisible(false);
        _buttons[BTN_RETREAT].setActive(false);
    }
    
    private void handleButton(int id)
    {
        int temp;
        Tank t;
//        System.out.println("Button id " + id + " clicked");

        switch (id)
        {
            case BTN_LEFT :
                t = _map._tanks[_curPlayer][_curTank];
                t.turnLeft();
                if (_map.isTankOnBase(_curPlayer, _curTank))
                    t.useMovement(-10);
                if (t.getMovement() < 1)
                    nextTank();
                _messageText = "Turning left";
                break;

            case BTN_RIGHT :
                t = _map._tanks[_curPlayer][_curTank];
                t.turnRight();
                if (_map.isTankOnBase(_curPlayer, _curTank))
                    t.useMovement(-10);
                if (t.getMovement() < 1)
                    nextTank();
                _messageText = "Turning right";
                break;

            case BTN_ADVANCE :
                _messageText = "Moving";
                t = _map._tanks[_curPlayer][_curTank];
                temp = _map.moveTankForward(_curPlayer, _curTank);
                if (temp < 0)
                    _messageText = "You can not move there!";
                else if (temp > 0)
                    _messageText = "You captured a factory!!!";
                if (t.getMovement() < 1)
                    nextTank();
                break;

            case BTN_RETREAT :
                System.out.println("Command no longer exists!");
                break;
                
            case BTN_FIRE :
                t = _map._tanks[_curPlayer][_curTank];
                temp = _map.fire(_curPlayer, _curTank);
                if (temp == -1)
                {
                    _messageText = "You do not have ammo!";
                }
                else if (temp == 1)
                {
                    _messageText = "You hit a tank";
                }
                else if (temp == 2)
                {
                    _messageText = "You destroyed a tank!";
                }
                else
                {
                    _messageText = "Shot never hit anything";
                }
                if (t.getMovement() < 1)
                    nextTank();
                break;
                
            case BTN_DONE :
                nextTank();
                break;
                
            case BTN_YES :
                _map.createTank(_curPlayer);
                _playerMoney[_curPlayer] -= _tankCost;
                _messageText = "Created tank!";
            case BTN_NO :
            case BTN_OK :
                hideDialog();
                nextTank();
                break;
                
            default :
                System.out.println("Unknown button pressed!");
        }
    }
    
    private void showDialog()
    {
        Container c = _displayHandler.getContainer();

        _dlgPlayer.setText(c, "Player #" + (_curPlayer + 1) );
        _dlgBase.setText(c, "Base earned " + _perTurnMoney);
        int temp = _map._playerFactoryCounts[_curPlayer];
        _dlgFactories.setText(c, "Factories: " + temp);
        _dlgTotal.setText(c, "Earning " + (temp * _factoryProfit));
        _dlgCash.setText(c, "You now have $" + _playerMoney[_curPlayer]);

        boolean showYN = false;
        boolean showOK = false;
        if (_map.canCreateTank(_curPlayer))
        {
            if (_playerMoney[_curPlayer] >= _tankCost)
            {
                _dlgBuild.setText(c, "Build new tank?");
                showYN = true;
            }
            else
            {
                _dlgBuild.setText(c, "Can\'t afford tank!");
                showOK = true;
            }
        }
        else
        {
            _dlgBuild.setText(c, "Base Occupied or Have 9 Tanks!");
            showOK = true;
        }
        _buttons[BTN_YES].setVisible(showYN);
        _buttons[BTN_NO].setVisible(showYN);
        _buttons[BTN_YES].setActive(showYN);
        _buttons[BTN_NO].setActive(showYN);
        _buttons[BTN_OK].setVisible(showOK);
        _buttons[BTN_OK].setActive(showOK);
        _showDialog = true;
        hideButtons();
    }
    
    private void hideDialog()
    {
        _buttons[BTN_YES].setVisible(false);
        _buttons[BTN_NO].setVisible(false);
        _buttons[BTN_OK].setVisible(false);
        _buttons[BTN_YES].setActive(false);
        _buttons[BTN_NO].setActive(false);
        _buttons[BTN_OK].setActive(false);
        _showDialog = false;
        showButtons();
    }


// ***********************************
// ***** METHODS - Game Handling *****
// ***********************************

    public void setPlayerType(int id, int type)
    {
        _playerType[id] = type;
    }

    public void startGame(int mapID)
    {
        int cntr;

        _messageText = "Starting Game";
        _gameOver = false;
        hideDialog();

        _map.cloneMap(_mapData[mapID]);
        for (cntr = 0; cntr < 4; ++cntr)
        {
            if (_playerType[cntr] > 0)
            {
                if ((_random.nextInt() & 1) > 0)
                {
                    _ai[cntr] = new TftMDumbAI(_map, cntr, _startingMoney, _perTurnMoney, _factoryProfit, _tankCost);
                    _displayHandler.log("player " + (cntr+1) + "Assigned AI type 1!");
                }
                else
                {
                    _ai[cntr] = new TftMSmartAI(_map, cntr, _startingMoney, _perTurnMoney, _factoryProfit, _tankCost);                
                    _displayHandler.log("player " + (cntr+1) + "Assigned AI type 2!");                    
                }
            }
        }

        // standard game initializatoin
        for (cntr = 0; cntr < 4; ++cntr)
            _playerMoney[cntr] = _startingMoney;
        _curDay = 1;
        _curPlayer = -1; // Cheating so will go to player 1 (id 0)
        nextPlayer();
//        showDialog();
    }

    private void nextTank()
    {
        boolean done = false;
        do
        {
            ++_curTank;
            if (_curTank >= 9)
                done = true;
            else if (_map._tanks[_curPlayer][_curTank].getHealth() > 0)
                done = true;
        }
        while ( ! done);
        if (_curTank >= 9)
        {
            nextPlayer();
            return;
        }
        else if (_playerType[_curPlayer] == 0)
        {
            // AI does this automatically, so only have to do it for humans
            _map._tanks[_curPlayer][_curTank].startTurn();
        }
    }
    
    /**
     * Checks to see if the game is over. If it isn't finds out who the next
     * player is and sets the game up to use that player.
     */
    private void nextPlayer()
    {
        ++_curPlayer;
        if (_curPlayer > 3)
        {
            int temp = _map.endDay();
            ++_curDay;
            if (temp >= 0)
            {
                _gameOver = true;
                _winner = temp;
                hideButtons();
                _displayHandler.refreshDisplay(this);
                playSound(WIN_SOUND);
                return;
            }
            _curPlayer = 0;
        }
        if (_playerType[_curPlayer] == 0) // is human?
            humanTurn();
        else
            computerTurn();
        
    }
    
    private void humanTurn()
    {
        _playerMoney[_curPlayer] += (_map._playerFactoryCounts[_curPlayer] * _factoryProfit + _perTurnMoney);
        _curTank = -1; // Make sure we start with the first tank (id 0)
        showDialog();
    }
    
    private void computerTurn()
    {
        hideButtons();
        _messageText = _ai[_curPlayer].startDay(_curDay);
        setRenderState(true);
    }

    private void setRenderState(boolean state)
    {
        _waitForRender = state;
    }
    
    private boolean getRenderState()
    {
        return _waitForRender;
    }
}
