//
//  TftMTitle.java
//  UlimateRetro
//
//  Created by spelchan on Mon Nov 19 2001.
//  Copyright (c) 2001 Spelchan Software. All rights reserved.
//

package com.blazinggames.retro.modern.search;

// Java imports
import java.awt.*;

// Spelchan imports
import com.blazinggames.urcl.ui.*;
import com.blazinggames.urcl.ui.comp.*;
import com.blazinggames.urcl.ui.sprite.*;

// Retro imports
import com.blazinggames.retro.modern.Title;


public class TftMTitle 
extends Title
{
// *********************
// ***** CONSTANTS *****
// *********************

    // Artillery title specific 
    private static final String PLAYERROWS = "12PLAYERROWS";
    private static final String PLAYERCOLS = "12PLAYERCOLS";

    // Shared constants (also used by ArtilleryDisplay)
    private static final String NUMPLAYERS = "12NUMPLAYERS";
    private static final String PLAYERCOLORS = "12PLAYERCOLOR";


// *********************
// ***** VARIABLES *****
// *********************

    protected int _playerRows[];
    protected int _playerCols[];
    protected SRadioButton _radioButtons[];
    protected SRadioContainer _playerContainers[];
    protected SLabel _playerLabels[];

    protected Rectangle _aiRect;
    

// ***********************
// ***** CONSTRUCTOR *****
// ***********************    

    public TftMTitle()
    {
        super();
    }


    public TftMTitle(String pref)
    {
        super(pref);
    }


// *****************************************
// ***** METHODS - Initialization aids *****
// *****************************************

    public void prepareRadioButtons()
    {
        Container c = _displayHandler.getContainer();
        Color pcolor;
        int n = 4;
        int temp, cntr;

        _aiRect = new Rectangle(_playerCols[0], _playerRows[0],
                                _playerCols[4] - _playerCols[0],
                                _playerRows[4] - _playerRows[0] );
        _radioButtons = new SRadioButton[n * 2];
        _playerContainers = new SRadioContainer[n];
        _playerLabels = new SLabel[n];

        for (cntr = 0; cntr < n; ++cntr) 
        {
            _playerContainers[cntr] = new SRadioContainer();
            _playerLabels[cntr] = new SLabel(c,  "Player " + (cntr + 1), 
					     _playerCols[0], _playerRows[cntr],
					     _playerCols[1]-_playerCols[0],
					     _playerRows[1]-_playerRows[0]);
            pcolor = _config.getColorProperty(PLAYERCOLORS + "[" + cntr + "]", Color.white);
            _playerLabels[cntr].setFontColor(pcolor);
            _radioButtons[cntr*2] = new SRadioButton(c, "Human", _playerCols[1], _playerRows[cntr]);
            _radioButtons[cntr*2].setForeground(pcolor);
            _radioButtons[cntr*2].setBackground(Color.black);
            _playerContainers[cntr].addRadioButton(_radioButtons[cntr*2]);
            _radioButtons[cntr*2+1] = new SRadioButton(c, "Computer", _playerCols[2], _playerRows[cntr]);
            _radioButtons[cntr*2+1].setForeground(pcolor);
            _radioButtons[cntr*2+1].setBackground(Color.black);
            _playerContainers[cntr].addRadioButton(_radioButtons[cntr*2+1]);
        }
        for (cntr = 1; cntr < n; ++cntr)
        {
            _playerContainers[cntr].setButton(1);
        }
    }


// **********************************
// ***** METHODS - From Display *****
// **********************************

    public boolean initialize(DisplayHandler dh)
    {
        super.initialize(dh);

        int cntr;
        String args[] = _config.parseString(_config.getStringProperty(PLAYERROWS, "0"), ',');
        _playerRows = new int[args.length];
        for (cntr = 0; cntr < args.length; ++cntr)
            _playerRows[cntr] = _config.parseInt(args[cntr], 0);

        args = _config.parseString(_config.getStringProperty(PLAYERCOLS, "0"), ',');
        _playerCols = new int[args.length];
        for (cntr = 0; cntr < args.length; ++cntr)
            _playerCols[cntr] = _config.parseInt(args[cntr], 0);

        prepareRadioButtons();
        return true;
    }
    
    public void mouse_move(int x, int y, boolean b)
    {
        boolean repaint = false;
        int cntr;

        for (cntr = 0; cntr < _buttons.length; ++cntr)
        {
            repaint |= _buttons[cntr].mouse_move(x, y, b);
        }
        for (cntr = 0; cntr < _playerContainers.length; ++cntr)
        {
            repaint |= _playerContainers[cntr].mouse_move(x, y, b);
        }
        
        if (repaint)
        {
            _displayHandler.refreshDisplay(this);
        }
    }
    
    public void mouse_down(int x, int y)
    {
        boolean repaint = false;
        int cntr;

        for (cntr = 0; cntr < _buttons.length; ++cntr)
        {
            repaint |= _buttons[cntr].mouse_down(x, y);
        }
        for (cntr = 0; cntr < _playerContainers.length; ++cntr)
        {
            repaint |= _playerContainers[cntr].mouse_down(x, y);
        }
        
        if (repaint)
        {
            _displayHandler.refreshDisplay(this);
        }
    }

    public void mouse_up(int x, int y)
    {
        boolean repaint = false;
        int cntr;

        for (cntr = 0; cntr < _buttons.length; ++cntr)
        {
            if (_buttons[cntr].mouse_up(x, y))
            {
                _displayHandler.displayEvent(this, _buttonActions[cntr]);
                repaint = true;
            }
        }
        for (cntr = 0; cntr < _playerContainers.length; ++cntr)
        {
            repaint |= _playerContainers[cntr].mouse_up(x, y);
        }
        
        if (repaint)
        {
            _displayHandler.refreshDisplay(this);
        }
    }
    
    public synchronized void render(Graphics g, Component comp)
    {
        super.render(g, comp);

        g.setColor(Color.black);
        g.fillRect(_aiRect.x, _aiRect.y, _aiRect.width, _aiRect.height);
        
        for (int cntr = 0; cntr < _playerContainers.length; ++cntr) 
        {
            _playerContainers[cntr].render(g, comp);
            _playerLabels[cntr].render(g, comp);
        }
    }


// ***************************************************************************
// ***** METHODS - Additional information getting (setting done by user) *****
// ***************************************************************************

    public int getPlayerType(int id)
    {
        return _playerContainers[id].getButton();
    }
}
