package com.blazinggames.retro.shared.search.tftm;

// Java imports
import java.awt.Point;
import java.util.Vector;

/**
 * Holds the path that the ai uses for pathfinding.
 *
 * @author Billy D. Spelchan
 * @version 0.50.011104
*/
public class Path 
{
// *********************
// ***** VARIABLES *****
// *********************

    protected Point _path[];
    protected int _cost;

// ************************
// ***** CONSTRUCTORs *****
// ************************

    public Path(Path copy)
    {
        int size = copy._path.length;
        
        _path = new Point[size];
        for (int cntr = 0; cntr < size; ++cntr)
            _path[cntr] = new Point(copy._path[cntr]);
        _cost = copy._cost;
    }

    public Path(Map map, Point start, Point end, int facing)
    {
        int width = map._mapWidth;
        int height = map._mapHeight;
        int cntr, cntrNode, cntrX, cntrY, numNodes, weight, attr, cost, best;
        int pathmap[][] = new int[width][height];
        int costmap[][] = new int[width][height];
        boolean stillSearching;
        
//System.out.println("Starting path at " + System.currentTimeMillis());
        // search initialization
        Vector oldNodes = new Vector();
        Vector newNodes = new Vector();
        Vector temp;
        for (cntrX = 0; cntrX < width; ++cntrX)
        {
            for (cntrY = 0; cntrY < height; ++cntrY)
            {
                attr = map._map.getAttribute(cntrX, cntrY);
                best = map._tileRenderer.getTravelSpeed(attr);
                if (attr >= MapTile.TILE_PLAYER1_BASE) best = 0;
                if ( ( attr >= MapTile.TILE_NEUTRAL_FACTORY) &&
                     ( attr <= MapTile.TILE_PLAYER4_FACTORY) )
                    best = 100;
                costmap[cntrX][cntrY] = best;
                if (best == 0)
                    weight = Integer.MIN_VALUE;
                else
                    weight = Integer.MAX_VALUE;
                pathmap[cntrX][cntrY] = weight;
            } 
        }
        // make sure start location wight 0 and dest cost is non-zero
        pathmap[start.x][start.y] = 0;
        costmap[end.x][end.y] = 10;
        newNodes.addElement(new PathPoint(start.x, start.y, facing, 0, null));

        // Now search the map
//System.out.println("Starting search at " + System.currentTimeMillis());
//int count = 0;
        PathPoint pp, bestNode = null;
        int x, y, turn, face;
        best = Integer.MAX_VALUE;
        while (newNodes.size() > 0)
        {
//++count;
//System.out.println("Pass " + count + " nodes " + newNodes.size()); 
            temp = oldNodes;
            oldNodes = newNodes;
            newNodes = temp;
            newNodes.removeAllElements();
            numNodes = oldNodes.size();
            for (cntrNode = 0; cntrNode < numNodes; ++cntrNode)
            {
                pp = (PathPoint)oldNodes.elementAt(cntrNode);
                if ( (pathmap[pp.x][pp.y] >= pp.weight) && (best > pp.weight) )
                {
                    face = pp.facing;
                    // unraveld loop here for speed
                    // Y-1 searches
                    y = pp.y - 1;
                    if (y >= 0)
                    {
                        // north west
                        if ((x = pp.x - 1) >= 0)
                        {
                            turn = 10 * (face < 3 ? face + 1 : 7 - face);
                            cost = pp.weight + turn + costmap[x][y];
                            if (cost < pathmap[x][y])
                            {
                                pathmap[x][y] = cost;
                                if ( (x == end.x) && (y == end.y) )
                                {
                                    bestNode = new PathPoint(x, y, 7, cost, pp);
                                    best = cost;
                                }
                                else
                                    newNodes.addElement(new PathPoint(x, y, 7, cost, pp));
                            }
                        }
                        
                        // north
                        x = pp.x;
                        turn = 10 * (face < 4 ? face : 8 - face);
                        cost = pp.weight + turn + costmap[x][y];
                        if (cost < pathmap[x][y])
                        {
                            pathmap[x][y] = cost;
                            if ( (x == end.x) && (y == end.y) )
                            {
                                bestNode = new PathPoint(x, y, 0, cost, pp);
                                best = cost;
                            }
                            else
                                newNodes.addElement(new PathPoint(x, y, 0, cost, pp));
                        }

                        // north east
                        if ((x = pp.x + 1) < width)
                        {
                            turn = 10 * (face < 5 ? (face == 0 ? 1 : face - 1) : 4 - face + 5);
                            cost = pp.weight + turn + costmap[x][y];
                            if (cost < pathmap[x][y])
                            {
                                pathmap[x][y] = cost;
                                if ( (x == end.x) && (y == end.y) )
                                {
                                    bestNode = new PathPoint(x, y, 1, cost, pp);
                                    best = cost;
                                }
                                else
                                    newNodes.addElement(new PathPoint(x, y, 1, cost, pp));
                            }
                        }

                    } // y-1 search ends
                    
                    // Y searches - we know y is valid so it's just x work
                    y = pp.y;
                    // west
                    if ((x = pp.x - 1) >= 0)
                    {
                        turn = 10 * (face < 6 ? (face > 2 ? 6 - face : face + 2) : face - 6);
                        cost = pp.weight + turn + costmap[x][y];
                        if (cost < pathmap[x][y])
                        {
                            pathmap[x][y] = cost;
                            if ( (x == end.x) && (y == end.y) )
                            {
                                bestNode = new PathPoint(x, y, 6, cost, pp);
                                best = cost;
                            }
                            else
                                newNodes.addElement(new PathPoint(x, y, 6, cost, pp));
                        }
                    }
                    
                    // East
                    if ((x = pp.x + 1) < width)
                    {
                        turn = 10 * (face == 7 ? 3 : face > 1 ? (face - 2) : 2 - face);
                        cost = pp.weight + turn + costmap[x][y];
                        if (cost < pathmap[x][y])
                        {
                            pathmap[x][y] = cost;
                            if ( (x == end.x) && (y == end.y) )
                            {
                                bestNode = new PathPoint(x, y, 2, cost, pp);
                                best = cost;
                            }
                            else
                                newNodes.addElement(new PathPoint(x, y, 2, cost, pp));
                        }
                    }
                    
                    // Y+1 searches
                    y = pp.y + 1;
                    if (y < height)
                    {
                        // south west
                        if ((x = pp.x - 1) >= 0)
                        {
                            turn = 10 * (face == 0 ? 3 : face < 5 ? 5 - face : face - 5);
                            cost = pp.weight + turn + costmap[x][y];
                            if (cost < pathmap[x][y])
                            {
                                pathmap[x][y] = cost;
                                if ( (x == end.x) && (y == end.y) )
                                {
                                    bestNode = new PathPoint(x, y, 5, cost, pp);
                                    best = cost;
                                }
                                else
                                    newNodes.addElement(new PathPoint(x, y, 5, cost, pp));
                            }
                        }
                        
                        // south
                        x = pp.x;
                        turn = 10 * ( face < 4 ? 4 - face : face - 4);
                        cost = pp.weight + turn + costmap[x][y];
                        if (cost < pathmap[x][y])
                        {
                            pathmap[x][y] = cost;
                            if ( (x == end.x) && (y == end.y) )
                            {
                                bestNode = new PathPoint(x, y, 4, cost, pp);
                                best = cost;
                            }
                            else
                                newNodes.addElement(new PathPoint(x, y, 4, cost, pp));
                        }

                        // south east
                        if ((x = pp.x + 1) < width)
                        {
                            turn = 10 * (face > 3 ? face - 3 : 3 - face);
                            cost = pp.weight + turn + costmap[x][y];
                            if (cost < pathmap[x][y])
                            {
                                pathmap[x][y] = cost;
                                if ( (x == end.x) && (y == end.y) )
                                {
                                    bestNode = new PathPoint(x, y, 3, cost, pp);
                                    best = cost;
                                }
                                else
                                    newNodes.addElement(new PathPoint(x, y, 3, cost, pp));
                            }
                        }

                    } // y+1 search ends

                }
            }
        }
        
        // convert the resulting map into a array of points
//System.out.println("Building path at " + System.currentTimeMillis());        
        temp = new Vector();
        temp.addElement(bestNode);
        pp = bestNode.parent;
        while (pp != null)
        {
            temp.addElement(pp);
            pp = pp.parent;
        }
        numNodes = temp.size();
        _path = new Point[numNodes];
        for (cntrNode = 0; cntrNode < numNodes; ++cntrNode)
        {
            pp = (PathPoint)temp.elementAt(cntrNode);
//System.out.println("node element " + (numNodes - cntrNode) + " is " + pp.x + "," + pp.y + " cost " + pp.weight);
            _path[numNodes - 1 - cntrNode] = new Point(pp.x, pp.y);
        }
        _cost = bestNode.weight;
//System.out.println("Finished path at " + System.currentTimeMillis());        
    }


// *********************************
// ***** METHODS - information *****
// *********************************

    public int getCost()
    {
        return _cost;
    }
    
    public Point getNextNode(Point p)
    {
        int noEndSize = _path.length - 1;
        
        for (int cntr = 0; cntr < noEndSize; ++cntr)
        {
            if (_path[cntr].equals(p))
                return new Point(_path[cntr+1]);
        }
        // if this is reached, then at end or not on path, so we just return current position
        return p;
    }
}
